Attribute VB_Name = "modRTF2XML"
Option Explicit

'Relevant RTF control words:
' \rtf1      -> start of header of RTF file
' \ansi      -> ANSI character set (default)
' \ansicpg   -> default ANSI code page (1252: Western European)
' \deff      -> sets the default font
' \deflang   -> default language used when the \plain control word is encountered (2067: Dutch (Belgium))
' \fonttbl   -> font table group
' \fnil      -> no specific font family (default)
' \fcharset  -> character set
' \colortbl  -> color table definitions as decimal RGB values (\red \green \blue)
' \viewkind  -> document view and zoom level (4: draft view)
' \viewscale -> zoom level of the document (argument is percentage, default: 100)
' \uc        -> number of bytes that follow a \u Unicode character
' \pard      -> reset to default paragraph properties
' \ql        -> left-aligned (default)
' \qc        -> centered
' \qr        -> right-aligned
' \qj        -> justified
' \lang      -> applies language to text run (2067: Dutch (Belgium))
' \cf        -> color foreground (index of entry in color table)
' \ul        -> underline (stop: \ulnone)
' \b         -> bold (stop: \b0)
' \i         -> italic (stop: \i0)
' \f         -> font number (refers to entry in font table)
' \fs        -> font size in half-points

Private Const SPECIAL_CHARS_TXT = "                                                                                               "
Private Const SPECIAL_CHARS_RTF = "\'a1 \'a2 \'a3 \'a4 \'a5 \'a6 \'a7 \'a8 \'a9 \'aa \'ab \'ac \'ae \'af \'b0 \'b1 \'b2 \'b3 \'b4 \'b5 \'b6 \'b7 \'b8 \'b9 \'ba \'bb \'bc \'bd \'be \'bf \'c0 \'c1 \'c2 \'c3 \'c4 \'c5 \'c6 \'c7 \'c8 \'c9 \'ca \'cb \'cc \'cd \'ce \'cf \'d0 \'d1 \'d2 \'d3 \'d4 \'d5 \'d6 \'d7 \'d8 \'d9 \'da \'db \'dc \'dd \'de \'df \'e0 \'e1 \'e2 \'e3 \'e4 \'e5 \'e6 \'e7 \'e8 \'e9 \'ea \'eb \'ec \'ed \'ee \'ef \'f0 \'f1 \'f2 \'f3 \'f4 \'f5 \'f6 \'f7 \'f8 \'f9 \'fa \'fb \'fc \'fd \'fe \'ff "
'Private Const SPECIAL_CHARS_XML = "¡ ¢ £ ¤ ¥ ¦ § ¨ © ª « ¬ ® ¯ ° ± ² ³ ´ µ ¶ · ¸ ¹ º » ¼ ½ ¾ ¿ À Á Â Ã Ä Å Æ Ç È É Ê Ë Ì Í Î Ï Ð Ñ Ò Ó Ô Õ Ö × Ø Ù Ú Û Ü Ý Þ ß à á â ã ä å æ ç è é ê ë ì í î ï ð ñ ò ó ô õ ö ÷ ø ù ú û ü ý þ ÿ "

Public Function RTF2XML(ByVal sRTF As String) As String
'Converts an RTF string to an XML string

  Dim sTemp      As String
  Dim sTemp2     As String
  Dim sSize      As String
  Dim sEmph      As String
  Dim sUnderline As String
  Dim sHAlign    As String
  Dim bBold      As Boolean
  Dim bItalic    As Boolean
  Dim arrFont()  As String
  Dim sFont      As String
  Dim arrColor() As String
  Dim sColor     As String
  Dim sColorR    As String
  Dim sColorG    As String
  Dim sColorB    As String
  Dim i          As Integer
  
  ReDim Preserve arrFont(0)
  arrFont(0) = vbNullString
  ReDim Preserve arrColor(0)
  arrColor(0) = vbNullString
  
  'Create font array:
  If InStr(sRTF, "{\fonttbl") Then
  
    sTemp = mid(sRTF, InStr(sRTF, "{\fonttbl") + 9)
    sTemp = Left(sTemp, InStr(sTemp, "}}"))
    
    Do While sTemp <> vbNullString
    
      ReDim Preserve arrFont(UBound(arrFont) + 1)
      arrFont(UBound(arrFont)) = mid(sTemp, InStr(sTemp, " ") + 1, InStr(sTemp, ";}") - InStr(sTemp, " ") - 1)
      sTemp = mid(sTemp, InStr(sTemp, ";}") + 2)
      
    Loop
    
  End If
  
  'Create color array:
  If InStr(sRTF, "{\colortbl ") Then
  
    sTemp = mid(sRTF, InStr(sRTF, "{\colortbl ") + 12)
    sTemp = Left(sTemp, InStr(sTemp, "}") - 1)
    
    Do While sTemp <> vbNullString
    
      ReDim Preserve arrColor(UBound(arrColor) + 1)
      sColor = Left(sTemp, InStr(sTemp, ";") - 1)
      sColorR = Hex(CInt(mid(sColor, 5, InStr(sColor, "\green") - 5)))
      sColorG = Hex(CInt(mid(sColor, InStr(sColor, "\green") + 6, InStr(sColor, "\blue") - InStr(sColor, "\green") - 6)))
      sColorB = Hex(CInt(mid(sColor, InStr(sColor, "\blue") + 5)))
      If Len(sColorR) = 1 Then sColorR = "0" & sColorR
      If Len(sColorG) = 1 Then sColorG = "0" & sColorG
      If Len(sColorB) = 1 Then sColorB = "0" & sColorB
      arrColor(UBound(arrColor)) = sColorR & sColorG & sColorB
      sTemp = mid(sTemp, InStr(sTemp, ";") + 1)
      
    Loop
    
  End If
  
  'Get general font:
  sFont = mid(sRTF, InStr(sRTF, "{\fonttbl{\f0\fnil\fcharset0 ") + 29)
  sFont = Left(sFont, InStr(sFont, ";") - 1)
  
  'Get general properties (alignment, color, underline, bold, italic, font size):
  sTemp = sRTF
  sTemp = mid(sTemp, InStr(sTemp, "\pard"))
  
  If IsDigit(mid(sTemp, InStr(sTemp, "\fs") + 3, 3)) Then
  
    sTemp = Left(sTemp, InStr(sTemp, "\fs") + 5)
    
  ElseIf IsDigit(mid(sTemp, InStr(sTemp, "\fs") + 3, 2)) Then
  
    sTemp = Left(sTemp, InStr(sTemp, "\fs") + 4)
    
  End If
  
  If InStr(sTemp, "\pard\qc") Then
  
    sHAlign = " HALIGN=" & Chr(34) & "CENTER" & Chr(34)
    
  ElseIf InStr(sTemp, "\pard\qr") Then
  
    sHAlign = " HALIGN=" & Chr(34) & "RIGHT" & Chr(34)
    
  ElseIf InStr(sTemp, "\pard\qj") Then
  
    sHAlign = " HALIGN=" & Chr(34) & "JUSTIFY" & Chr(34)
    
  Else
  
    sHAlign = " HALIGN=" & Chr(34) & "LEFT" & Chr(34)
    
  End If
  
  If InStr(sTemp, "\cf1") Then
  
    sColor = " COLOR=" & Chr(34) & arrColor(1) & Chr(34)
    
  Else
  
    sColor = " COLOR=" & Chr(34) & "000000" & Chr(34)
    
  End If
  
  If InStr(sTemp, "\ul") Then
  
    sUnderline = " UNDERLINE=" & Chr(34) & "1" & Chr(34)
    
  Else
  
    sUnderline = " UNDERLINE=" & Chr(34) & "0" & Chr(34)
    
  End If
  
  If InStr(sTemp, "\b") Then bBold = True
  If InStr(sTemp, "\i") Then bItalic = True
  
  If bBold And bItalic Then
  
    sEmph = " EMPH=" & Chr(34) & "BOLDITALIC" & Chr(34)
    
  ElseIf bBold Then
  
    sEmph = " EMPH=" & Chr(34) & "BOLD" & Chr(34)
    
  ElseIf bItalic Then
  
    sEmph = " EMPH=" & Chr(34) & "ITALIC" & Chr(34)
    
  Else
  
    sEmph = " EMPH=" & Chr(34) & Chr(34)
    
  End If
  
  sSize = mid(sTemp, InStr(sTemp, "\fs") + 3)
  
  'Convert first paragraph:
  sTemp = sRTF
  sTemp = mid(sTemp, InStr(sTemp, "\pard"))
  
  If IsDigit(mid(sTemp, InStr(sTemp, "\fs") + 3, 3)) Then
  
    sTemp = mid(sTemp, InStr(sTemp, "\fs") + 6)
    
  ElseIf IsDigit(mid(sTemp, InStr(sTemp, "\fs") + 3, 2)) Then
  
    sTemp = mid(sTemp, InStr(sTemp, "\fs") + 5)
    
  End If
  
  'Convert last paragraph:
  sTemp = Replace(sTemp, vbCrLf & "\par }" & vbCrLf, vbNullString)
  
  'Convert other paragraphs (RETURN_BLANK for empty line / RETURN for next line):
  sTemp = Replace(sTemp, vbCrLf & "\par ", "<RETURN_BLANK/>")
  
  If InStr(sTemp, "<RETURN_BLANK/>") Then
  
    sTemp2 = sTemp
    sTemp = vbNullString
    
    Do While InStr(sTemp2, "<RETURN_BLANK/>") = 1
    
      sTemp = sTemp & "<RETURN_BLANK/>"
      sTemp2 = mid(sTemp2, 16)
      
    Loop
    
    Do While sTemp2 <> vbNullString
    
      If InStr(sTemp2, "<RETURN_BLANK/>") Then
      
        sTemp = sTemp & Left(sTemp2, InStr(sTemp2, "<RETURN_BLANK/>") - 1) & "<RETURN/>"
        sTemp2 = mid(sTemp2, InStr(sTemp2, "<RETURN_BLANK/>") + 15)
        
        Do While InStr(sTemp2, "<RETURN_BLANK/>") = 1
        
          sTemp = sTemp & "<RETURN_BLANK/>"
          sTemp2 = mid(sTemp2, 16)
          
        Loop
        
      Else
      
        sTemp = sTemp & sTemp2
        sTemp2 = vbNullString
        
      End If
      
    Loop
    
  End If
  
  'Convert alignment:
  sTemp = Replace(sTemp, "\pard\qc ", "<FONT ALIGNMENT=" & Chr(34) & "CENTER" & Chr(34) & "/>")
  sTemp = Replace(sTemp, "\pard\qc", "<FONT ALIGNMENT=" & Chr(34) & "CENTER" & Chr(34) & "/>")
  sTemp = Replace(sTemp, "\pard\qr ", "<FONT ALIGNMENT=" & Chr(34) & "RIGHT" & Chr(34) & "/>")
  sTemp = Replace(sTemp, "\pard\qr", "<FONT ALIGNMENT=" & Chr(34) & "RIGHT" & Chr(34) & "/>")
  sTemp = Replace(sTemp, "\pard\qj ", "<FONT ALIGNMENT=" & Chr(34) & "JUSTIFY" & Chr(34) & "/>")
  sTemp = Replace(sTemp, "\pard\qj", "<FONT ALIGNMENT=" & Chr(34) & "JUSTIFY" & Chr(34) & "/>")
  sTemp = Replace(sTemp, "\pard ", "<FONT ALIGNMENT=" & Chr(34) & "LEFT" & Chr(34) & "/>")
  sTemp = Replace(sTemp, "\pard", "<FONT ALIGNMENT=" & Chr(34) & "LEFT" & Chr(34) & "/>")
  
  'Convert color:
  sTemp = Replace(sTemp, "\cf0 ", "<FONT COLOR=" & Chr(34) & "000000" & Chr(34) & "/>")
  sTemp = Replace(sTemp, "\cf0", "<FONT COLOR=" & Chr(34) & "000000" & Chr(34) & "/>")
  
  For i = 1 To UBound(arrColor)
  
    sTemp = Replace(sTemp, "\cf" & i & " ", "<FONT COLOR=" & Chr(34) & arrColor(i) & Chr(34) & "/>")
    sTemp = Replace(sTemp, "\cf" & i, "<FONT COLOR=" & Chr(34) & arrColor(i) & Chr(34) & "/>")
    
  Next
  
  'Convert underline, bold, italic:
  sTemp = Replace(sTemp, "\ul\b\i0 ", "<U1/><B1/><I0/>")
  sTemp = Replace(sTemp, "\ul\b\i ", "<U1/><B1/><I1/>")
  sTemp = Replace(sTemp, "\ul\b0\i0 ", "<U1/><B0/><I0/>")
  sTemp = Replace(sTemp, "\ul\b0\i ", "<U1/><B0/><I1/>")
  sTemp = Replace(sTemp, "\ulnone\b\i0 ", "<U0/><B1/><I0/>")
  sTemp = Replace(sTemp, "\ulnone\b\i ", "<U0/><B1/><I1/>")
  sTemp = Replace(sTemp, "\ulnone\b0\i0 ", "<U0/><B0/><I0/>")
  sTemp = Replace(sTemp, "\ulnone\b0\i ", "<U0/><B0/><I1/>")
  sTemp = Replace(sTemp, "\b\i0 ", "<B1/><I0/>")
  sTemp = Replace(sTemp, "\b\i ", "<B1/><I1/>")
  sTemp = Replace(sTemp, "\b0\i0 ", "<B0/><I0/>")
  sTemp = Replace(sTemp, "\b0\i ", "<B0/><I1/>")
  sTemp = Replace(sTemp, "\ul\i0 ", "<U1/><I0/>")
  sTemp = Replace(sTemp, "\ul\i ", "<U1/><I1/>")
  sTemp = Replace(sTemp, "\ulnone\i0 ", "<U0/><I0/>")
  sTemp = Replace(sTemp, "\ulnone\i ", "<U0/><I1/>")
  sTemp = Replace(sTemp, "\ul\b0 ", "<U1/><B0/>")
  sTemp = Replace(sTemp, "\ul\b ", "<U1/><B1/>")
  sTemp = Replace(sTemp, "\ulnone\b0 ", "<U0/><B0/>")
  sTemp = Replace(sTemp, "\ulnone\b ", "<U0/><B1/>")
  sTemp = Replace(sTemp, "\ulnone ", "<U0/>")
  sTemp = Replace(sTemp, "\ul ", "<U1/>")
  sTemp = Replace(sTemp, "\b0 ", "<B0/>")
  sTemp = Replace(sTemp, "\b ", "<B1/>")
  sTemp = Replace(sTemp, "\i0 ", "<I0/>")
  sTemp = Replace(sTemp, "\i ", "<I1/>")
  sTemp = Replace(sTemp, "\ul\b\i0", "<U1/><B1/><I0/>")
  sTemp = Replace(sTemp, "\ul\b\i", "<U1/><B1/><I1/>")
  sTemp = Replace(sTemp, "\ul\b0\i0", "<U1/><B0/><I0/>")
  sTemp = Replace(sTemp, "\ul\b0\i", "<U1/><B0/><I1/>")
  sTemp = Replace(sTemp, "\ulnone\b\i0", "<U0/><B1/><I0/>")
  sTemp = Replace(sTemp, "\ulnone\b\i", "<U0/><B1/><I1/>")
  sTemp = Replace(sTemp, "\ulnone\b0\i0", "<U0/><B0/><I0/>")
  sTemp = Replace(sTemp, "\ulnone\b0\i", "<U0/><B0/><I1/>")
  sTemp = Replace(sTemp, "\b\i0", "<B1/><I0/>")
  sTemp = Replace(sTemp, "\b\i", "<B1/><I1/>")
  sTemp = Replace(sTemp, "\b0\i0", "<B0/><I0/>")
  sTemp = Replace(sTemp, "\b0\i", "<B0/><I1/>")
  sTemp = Replace(sTemp, "\ul\i0", "<U1/><I0/>")
  sTemp = Replace(sTemp, "\ul\i", "<U1/><I1/>")
  sTemp = Replace(sTemp, "\ulnone\i0", "<U0/><I0/>")
  sTemp = Replace(sTemp, "\ulnone\i", "<U0/><I1/>")
  sTemp = Replace(sTemp, "\ul\b0", "<U1/><B0/>")
  sTemp = Replace(sTemp, "\ul\b", "<U1/><B1/>")
  sTemp = Replace(sTemp, "\ulnone\b0", "<U0/><B0/>")
  sTemp = Replace(sTemp, "\ulnone\b", "<U0/><B1/>")
  sTemp = Replace(sTemp, "\ulnone", "<U0/>")
  sTemp = Replace(sTemp, "\ul", "<U1/>")
  sTemp = Replace(sTemp, "\b0", "<B0/>")
  sTemp = Replace(sTemp, "\b", "<B1/>")
  sTemp = Replace(sTemp, "\i0", "<I0/>")
  sTemp = Replace(sTemp, "\i", "<I1/>")
  
  'Convert font:
  sTemp = Replace(sTemp, "\f0 ", "<FONT NAME=" & Chr(34) & sFont & Chr(34) & "/>")
  sTemp = Replace(sTemp, "\f0", "<FONT NAME=" & Chr(34) & sFont & Chr(34) & "/>")
  
  For i = 1 To UBound(arrFont) - 1
  
    sTemp = Replace(sTemp, "\f" & i & " ", "<FONT NAME=" & Chr(34) & arrFont(i + 1) & Chr(34) & "/>")
    sTemp = Replace(sTemp, "\f" & i, "<FONT NAME=" & Chr(34) & arrFont(i + 1) & Chr(34) & "/>")
    
  Next
  
  'Convert font size:
  
  Dim sTmpSize As String
  
  sTemp2 = sTemp
  sTemp = vbNullString
  
  Do While InStr(sTemp2, "\fs")
  
    sTmpSize = mid(sTemp2, InStr(sTemp2, "\fs") + 3)
    sTmpSize = Left(sTmpSize, InStr(sTmpSize, " ") - 1)
    sTemp = sTemp & Left(sTemp2, InStr(sTemp2, "\fs") - 1) & "<FONT SIZE=" & Chr(34) & Replace(CStr(CInt(sTmpSize) / 2), ",", ".") & "pt" & Chr(34) & "/>"
    sTemp2 = mid(sTemp2, InStr(sTemp2, "\fs"))
    sTemp2 = mid(sTemp2, InStr(sTemp2, " ") + 1)
    
  Loop
  
  sTemp = sTemp & sTemp2
  
  'Delete trailing tags:
  
  Do While Right(sTemp, 2) = "/>"
  
    sTemp2 = StrReverse(sTemp)
    sTemp = StrReverse(mid(sTemp2, InStr(sTemp2, "<") + 1))
    
  Loop
  
  'XML:
  RTF2XML = "<TEXT FONT=" & Chr(34) & sFont & Chr(34) & " SIZE=" & Chr(34) & Replace(CStr(CInt(sSize) / 2), ",", ".") & "pt" & Chr(34) & sEmph & sUnderline & sHAlign & sColor & ">" & ConvertSpecialChars_RTF2TXT(sTemp) & "</TEXT>"
  
End Function

Public Function XML2RTF(ByVal sXML As String, Optional ByVal fFontSizeResz As Single = 1) As String
'Converts an XML string to an RTF string, font sizes are correctly scaled if necessary

  Dim sTemp      As String
  Dim sSize      As String
  Dim sEmph      As String
  Dim sUnderline As String
  Dim sHAlign    As String
  Dim sBold      As String
  Dim sItalic    As String
  Dim arrFont()  As String
  Dim sFontTbl   As String
  Dim sFont      As String
  Dim arrColor() As String
  Dim sColorTbl  As String
  Dim sColor     As String
  Dim i          As Integer
  
  ReDim Preserve arrFont(0)
  arrFont(0) = vbNullString
  ReDim Preserve arrColor(0)
  arrColor(0) = vbNullString
  
  'Get general font:
  sFont = mid(sXML, InStr(sXML, " FONT=") + 7)
  sFont = Left(sFont, InStr(sFont, Chr(34)) - 1)
  
  'Create font array:
  If InStr(sXML, "&lt;FONT NAME=" & Chr(34)) Then
  
    sTemp = sXML
    
    Do While InStr(sTemp, "&lt;FONT NAME=" & Chr(34))
    
      sTemp = mid(sTemp, InStr(sTemp, "&lt;FONT NAME=" & Chr(34)) + 15)
      
      If Left(sTemp, InStr(sTemp, Chr(34)) - 1) <> sFont Then
      
        ReDim Preserve arrFont(UBound(arrFont) + 1)
        arrFont(UBound(arrFont)) = Left(sTemp, InStr(sTemp, Chr(34)) - 1)
        
      End If
      
      sTemp = mid(sTemp, InStr(sTemp, "/&gt;") + 5)
      
    Loop
    
  End If
  
  'Create font table:
  For i = 1 To UBound(arrFont)
  
    sFontTbl = sFontTbl & "{\f" & i & "\fnil\fcharset0 " & arrFont(i) & ";}"
    
  Next
  
  'Get general color:
  sColor = mid(sXML, InStr(sXML, " COLOR=") + 8)
  sColor = Left(sColor, InStr(sColor, Chr(34)) - 1)
  
  'Create color array:
  
  If sColor <> "000000" Then
  
    ReDim Preserve arrColor(UBound(arrColor) + 1)
    arrColor(UBound(arrColor)) = sColor
    
  End If
  
  If InStr(sXML, "&lt;FONT COLOR=" & Chr(34)) Then
  
    sTemp = sXML
    
    Do While InStr(sTemp, "&lt;FONT COLOR=" & Chr(34))
    
      sTemp = mid(sTemp, InStr(sTemp, "&lt;FONT COLOR=" & Chr(34)) + 16)
      
      If Left(sTemp, InStr(sTemp, Chr(34)) - 1) <> sColor Then
      
        ReDim Preserve arrColor(UBound(arrColor) + 1)
        arrColor(UBound(arrColor)) = Left(sTemp, InStr(sTemp, Chr(34)) - 1)
        
      End If
      
      sTemp = mid(sTemp, InStr(sTemp, "/&gt;") + 5)
      
    Loop
    
  End If
  
  'Create color table:
  If UBound(arrColor) > 0 Then
  
    sColorTbl = "{\colortbl ;"
    
    For i = 1 To UBound(arrColor)
    
      sColorTbl = sColorTbl & "\red" & HextoLong(Left(arrColor(i), 2)) & "\green" & HextoLong(mid(arrColor(i), 3, 2)) & "\blue" & HextoLong(mid(arrColor(i), 5, 2)) & ";"
      
    Next
    
    sColorTbl = sColorTbl & "}" & vbCrLf
    
  End If
  
  'Get general properties (alignment, color, underline, bold, italic, font size):
  sTemp = Left(sXML, InStr(sXML, ">"))
  
  If InStr(sTemp, " HALIGN=" & Chr(34) & "CENTER" & Chr(34)) Then
  
    sHAlign = "\pard\qc"
    
  ElseIf InStr(sTemp, " HALIGN=" & Chr(34) & "RIGHT" & Chr(34)) Then
  
    sHAlign = "\pard\qr"
    
  ElseIf InStr(sTemp, " HALIGN=" & Chr(34) & "JUSTIFY" & Chr(34)) Then
  
    sHAlign = "\pard\qj"
    
  Else
  
    sHAlign = "\pard"
    
  End If
  
  If InStr(sTemp, " COLOR=" & Chr(34) & "000000" & Chr(34)) Then
  
    sColor = vbNullString
    
  Else
  
    sColor = "\cf1"
    
  End If
  
  If InStr(sTemp, " UNDERLINE=" & Chr(34) & "1" & Chr(34)) Then sUnderline = "\ul"
  
  If InStr(sTemp, " EMPH=" & Chr(34) & "BOLDITALIC" & Chr(34)) Then
  
    sBold = "\b"
    sItalic = "\i"
    
  ElseIf InStr(sTemp, " EMPH=" & Chr(34) & "BOLD" & Chr(34)) Then
  
    sBold = "\b"
    
  ElseIf InStr(sTemp, " EMPH=" & Chr(34) & "ITALIC" & Chr(34)) Then
  
    sItalic = "\i"
    
  End If
  
  sSize = mid(sTemp, InStr(sTemp, " SIZE=") + 7)
  sSize = Left(sSize, InStr(sSize, Chr(34)) - 1)
  If Not (bDecPoint) Then sSize = Replace(sSize, ".", ",")
  
  'Convert TEXT tags:
  sTemp = mid(sXML, InStr(sXML, ">") + 1)
  sTemp = Replace(sTemp, "</TEXT>", vbNullString)
  
  'Convert paragraphs:
  sTemp = Replace(sTemp, "&lt;RETURN_BLANK/&gt;", vbCrLf & "\par ")
  sTemp = Replace(sTemp, "&lt;RETURN/&gt;", vbCrLf & "\par ")
  
  'Convert alignment:
  sTemp = Replace(sTemp, "&lt;FONT ALIGNMENT=" & Chr(34) & "CENTER" & Chr(34) & "/&gt;", "\pard\qc ")
  sTemp = Replace(sTemp, "&lt;FONT ALIGNMENT=" & Chr(34) & "RIGHT" & Chr(34) & "/&gt;", "\pard\qr ")
  sTemp = Replace(sTemp, "&lt;FONT ALIGNMENT=" & Chr(34) & "JUSTIFY" & Chr(34) & "/&gt;", "\pard\qj ")
  sTemp = Replace(sTemp, "&lt;FONT ALIGNMENT=" & Chr(34) & "LEFT" & Chr(34) & "/&gt;", "\pard ")
  
  'Convert color:
  sTemp = Replace(sTemp, "&lt;FONT COLOR=" & Chr(34) & "000000" & Chr(34) & "/&gt;", "\cf0 ")
  
  For i = 1 To UBound(arrColor)
  
    sTemp = Replace(sTemp, "&lt;FONT COLOR=" & Chr(34) & arrColor(i) & Chr(34) & "/&gt;", "\cf" & i & " ")
    
  Next
  
  'Convert underline, bold, italic:
  sTemp = Replace(sTemp, "&lt;U1/&gt;&lt;B1/&gt;&lt;I1/&gt;", "\ul\b\i ")
  sTemp = Replace(sTemp, "&lt;U1/&gt;&lt;B1/&gt;&lt;I0/&gt;", "\ul\b\i0 ")
  sTemp = Replace(sTemp, "&lt;U1/&gt;&lt;B0/&gt;&lt;I1/&gt;", "\ul\b0\i ")
  sTemp = Replace(sTemp, "&lt;U1/&gt;&lt;B0/&gt;&lt;I0/&gt;", "\ul\b0\i0 ")
  sTemp = Replace(sTemp, "&lt;U0/&gt;&lt;B1/&gt;&lt;I1/&gt;", "\ulnone\b\i ")
  sTemp = Replace(sTemp, "&lt;U0/&gt;&lt;B1/&gt;&lt;I0/&gt;", "\ulnone\b\i0 ")
  sTemp = Replace(sTemp, "&lt;U0/&gt;&lt;B0/&gt;&lt;I1/&gt;", "\ulnone\b0\i ")
  sTemp = Replace(sTemp, "&lt;U0/&gt;&lt;B0/&gt;&lt;I0/&gt;", "\ulnone\b0\i0 ")
  sTemp = Replace(sTemp, "&lt;B1/&gt;&lt;I1/&gt;", "\b\i ")
  sTemp = Replace(sTemp, "&lt;B1/&gt;&lt;I0/&gt;", "\b\i0 ")
  sTemp = Replace(sTemp, "&lt;B0/&gt;&lt;I1/&gt;", "\b0\i ")
  sTemp = Replace(sTemp, "&lt;B0/&gt;&lt;I0/&gt;", "\b0\i0 ")
  sTemp = Replace(sTemp, "&lt;U1/&gt;&lt;I1/&gt;", "\ul\i ")
  sTemp = Replace(sTemp, "&lt;U1/&gt;&lt;I0/&gt;", "\ul\i0 ")
  sTemp = Replace(sTemp, "&lt;U0/&gt;&lt;I1/&gt;", "\ulnone\i ")
  sTemp = Replace(sTemp, "&lt;U0/&gt;&lt;I0/&gt;", "\ulnone\i0 ")
  sTemp = Replace(sTemp, "&lt;U1/&gt;&lt;B1/&gt;", "\ul\b ")
  sTemp = Replace(sTemp, "&lt;U1/&gt;&lt;B0/&gt;", "\ul\b0 ")
  sTemp = Replace(sTemp, "&lt;U0/&gt;&lt;B1/&gt;", "\ulnone\b ")
  sTemp = Replace(sTemp, "&lt;U0/&gt;&lt;B0/&gt;", "\ulnone\b0 ")
  sTemp = Replace(sTemp, "&lt;U0/&gt;", "\ulnone ")
  sTemp = Replace(sTemp, "&lt;U1/&gt;", "\ul ")
  sTemp = Replace(sTemp, "&lt;B0/&gt;", "\b0 ")
  sTemp = Replace(sTemp, "&lt;B1/&gt;", "\b ")
  sTemp = Replace(sTemp, "&lt;I0/&gt;", "\i0 ")
  sTemp = Replace(sTemp, "&lt;I1/&gt;", "\i ")
  
  'Convert font:
  sTemp = Replace(sTemp, "&lt;FONT NAME=" & Chr(34) & sFont & Chr(34) & "/&gt;", "\f0 ")
  
  For i = 1 To UBound(arrFont)
  
    sTemp = Replace(sTemp, "&lt;FONT NAME=" & Chr(34) & arrFont(i) & Chr(34) & "/&gt;", "\f" & i & " ")
    
  Next
  
  'Convert font size:
  
  Dim sTemp2   As String
  Dim sTmpSize As String
  
  sTemp2 = sTemp
  sTemp = vbNullString
  
  Do While InStr(sTemp2, "&lt;FONT SIZE=")
  
    sTmpSize = mid(sTemp2, InStr(sTemp2, "&lt;FONT SIZE=") + 15)
    sTmpSize = Left(sTmpSize, InStr(sTmpSize, "pt") - 1)
    If Not (bDecPoint) Then sTmpSize = Replace(sTmpSize, ".", ",")
    sTemp = sTemp & Left(sTemp2, InStr(sTemp2, "&lt;FONT SIZE=") - 1) & "\fs" & Round(2 * fFontSizeResz * sTmpSize) & " "
    sTemp2 = mid(sTemp2, InStr(sTemp2, "&lt;FONT SIZE="))
    sTemp2 = mid(sTemp2, InStr(sTemp2, "&gt;") + 4)
    
  Loop
  
  sTemp = sTemp & sTemp2
  
  'RTF:
  XML2RTF = "{\rtf1\deff0{\fonttbl{\f0\fnil\fcharset0 " & sFont & ";}" & sFontTbl & "}" & vbCrLf & sColorTbl & sHAlign & sColor & sUnderline & sBold & sItalic & "\fs" & Round(2 * fFontSizeResz * Left(sSize, Len(sSize) - 2)) & " " & ConvertSpecialChars_TXT2RTF(sTemp) & vbCrLf & "\par }" & vbCrLf
  
End Function

Private Function HextoLong(HexNum As String) As Long

  HextoLong = Val("&h" & HexNum$)
  
End Function

'Private Function ConvertSpecialChars_RTF2XML(ByVal sString As String) As String
'Converts an RTF encoded special character string to an XML encoded special character string

'  Dim i As Integer
  
'  For i = 1 To Len(SPECIAL_CHARS_RTF) / 5
  
'    sString = Replace(sString, mid(SPECIAL_CHARS_RTF, 5 * i - 4, 4), mid(SPECIAL_CHARS_XML, 3 * i - 2, 2))
    
'  Next
  
'  ConvertSpecialChars_RTF2XML = Replace(sString, "\'80", "<EURO/>")
  
'End Function

Private Function ConvertSpecialChars_RTF2TXT(ByVal sString As String) As String
'Converts an RTF encoded special character string to a TXT special character string

  Dim i As Integer
  
  For i = 1 To Len(SPECIAL_CHARS_RTF) / 5
  
    sString = Replace(sString, mid(SPECIAL_CHARS_RTF, 5 * i - 4, 4), mid(SPECIAL_CHARS_TXT, 2 * i - 1, 1))
    
  Next
  
  ConvertSpecialChars_RTF2TXT = Replace(sString, "\'80", "<EURO/>")
  
End Function

'Private Function ConvertSpecialChars_XML2RTF(ByVal sString As String) As String
'Converts an XML encoded special character string to an RTF encoded special character string

'  Dim i As Integer
  
'  For i = 1 To Len(SPECIAL_CHARS_XML) / 3
  
'    sString = Replace(sString, mid(SPECIAL_CHARS_XML, 3 * i - 2, 2), mid(SPECIAL_CHARS_RTF, 5 * i - 4, 4))
    
'  Next
  
'  ConvertSpecialChars_XML2RTF = Replace(sString, "&lt;EURO/&gt;", "\'80")
  
'End Function

Private Function ConvertSpecialChars_TXT2RTF(ByVal sString As String) As String
'Converts a TXT special character string to an RTF encoded special character string

  Dim i As Integer
  
  For i = 1 To Len(SPECIAL_CHARS_TXT) / 3
  
    sString = Replace(sString, mid(SPECIAL_CHARS_TXT, 2 * i - 1, 1), mid(SPECIAL_CHARS_RTF, 5 * i - 4, 4))
    
  Next
  
  ConvertSpecialChars_TXT2RTF = Replace(Replace(sString, "&amp;", "&"), "&lt;EURO/&gt;", "\'80")
  
End Function
